/***********************************************************************
*
* Copyright (c) 2021-2022 Barbara Geller and Ansel Sermersheim
*
* This example is free software, released under the BSD 2-Clause license.
* For license details refer to LICENSE provided with this project.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* https://opensource.org/licenses/BSD-2-Clause
*
***********************************************************************/

#include <QtCore>
#include <QtGui>

class DrawArea;

class MainWindow : public QWidget
{
 public:
   MainWindow();

 private:
   DrawArea *renderArea;

   QComboBox *penCB;
   QComboBox *brushCB;

   void penChanged();
   void brushChanged();
};

class DrawArea : public QWidget
{
   CS_OBJECT(DrawArea)

 public:
   DrawArea(QWidget *parent = nullptr);

   void setPen(const QPen &pen);
   void setBrush(const QBrush &brush);

 protected:
   void paintEvent(QPaintEvent *event);

 private:
   QPen pen;
   QBrush brush;
};

MainWindow::MainWindow()
{
   setMinimumSize(700, 500);

   renderArea = new DrawArea;

   //
   QLabel *penLabel = new QLabel("Pen:");

   penCB = new QComboBox;
   penCB->addItem("Solid",               QVariant::fromValue(Qt::SolidLine));
   penCB->addItem("Dash",                QVariant::fromValue(Qt::DashLine));
   penCB->addItem("Dot",                 QVariant::fromValue(Qt::DotLine));
   penCB->addItem("Dash Dot",            QVariant::fromValue(Qt::DashDotLine));
   penCB->addItem("Dash Dot Dot",        QVariant::fromValue(Qt::DashDotDotLine));
   penCB->addItem("None",                QVariant::fromValue(Qt::NoPen));

   //
   QLabel *brushLabel = new QLabel("Brush:");

   brushCB = new QComboBox;
   brushCB->addItem("Linear Gradient",   QVariant::fromValue(Qt::LinearGradientPattern));
   brushCB->addItem("Radial Gradient",   QVariant::fromValue(Qt::RadialGradientPattern));
   brushCB->addItem("Conical Gradient",  QVariant::fromValue(Qt::ConicalGradientPattern));
   brushCB->addItem("Solid",             QVariant::fromValue(Qt::SolidPattern));
   brushCB->addItem("Horizontal",        QVariant::fromValue(Qt::HorPattern));
   brushCB->addItem("Vertical",          QVariant::fromValue(Qt::VerPattern));
   brushCB->addItem("Cross",             QVariant::fromValue(Qt::CrossPattern));
   brushCB->addItem("Backward Diagonal", QVariant::fromValue(Qt::BDiagPattern));
   brushCB->addItem("Forward Diagonal",  QVariant::fromValue(Qt::FDiagPattern));
   brushCB->addItem("Diagonal Cross",    QVariant::fromValue(Qt::DiagCrossPattern));
   brushCB->addItem("None",              QVariant::fromValue(Qt::NoBrush));

   QPushButton *pb_close = new QPushButton();
   pb_close->setText("Close");

   QGridLayout *layout_grid = new QGridLayout;

   layout_grid->setColumnStretch(2, 1);
   layout_grid->setHorizontalSpacing(20);
   layout_grid->setVerticalSpacing(15);

   layout_grid->addWidget(penLabel,   2, 0);
   layout_grid->addWidget(penCB,      2, 1);
   layout_grid->addWidget(brushLabel, 3, 0);
   layout_grid->addWidget(brushCB,    3, 1);

   QHBoxLayout *layout_pb = new QHBoxLayout();
   layout_pb->addStretch();
   layout_pb->addWidget(pb_close);
   layout_pb->addStretch();

   QVBoxLayout *layoutMain = new QVBoxLayout(this);
   layoutMain->setContentsMargins(40, 35, 40, 15);

   layoutMain->addWidget(renderArea);
   layoutMain->addSpacing(25);
   layoutMain->addLayout(layout_grid);
   layoutMain->addSpacing(25);
   layoutMain->addLayout(layout_pb);

   connect(penCB,   cs_mp_cast<int>(&QComboBox::activated), this, &MainWindow::penChanged);
   connect(brushCB, cs_mp_cast<int>(&QComboBox::activated), this, &MainWindow::brushChanged);

   connect(pb_close, &QPushButton::clicked, this, &QWidget::close);

   penChanged();
   brushChanged();
}

void MainWindow::penChanged()
{
   int width = 4;

   Qt::PenStyle style    = penCB->itemData(penCB->currentIndex(), Qt::UserRole).value<Qt::PenStyle>();
   Qt::PenCapStyle cap   = Qt::PenCapStyle::RoundCap;
   Qt::PenJoinStyle join = Qt::PenJoinStyle::RoundJoin;

   renderArea->setPen(QPen(Qt::blue, width, style, cap, join));
}

void MainWindow::brushChanged()
{
   Qt::BrushStyle style = brushCB->itemData(brushCB->currentIndex(), Qt::UserRole).value<Qt::BrushStyle>();

   if (style == Qt::LinearGradientPattern) {
     QLinearGradient linearGradient(0, 0, 300, 300);
     linearGradient.setColorAt(0.0, Qt::white);
     linearGradient.setColorAt(0.2, Qt::green);
     linearGradient.setColorAt(1.0, Qt::black);

     renderArea->setBrush(linearGradient);

   } else if (style == Qt::RadialGradientPattern) {
     QRadialGradient radialGradient(100, 100, 150, 210, 210);
     radialGradient.setColorAt(0.0, Qt::white);
     radialGradient.setColorAt(0.2, Qt::green);
     radialGradient.setColorAt(1.0, Qt::black);

     renderArea->setBrush(radialGradient);

   } else if (style == Qt::ConicalGradientPattern) {
     QConicalGradient conicalGradient(150, 150, 150);
     conicalGradient.setColorAt(0.0, Qt::white);
     conicalGradient.setColorAt(0.2, Qt::green);
     conicalGradient.setColorAt(1.0, Qt::black);

     renderArea->setBrush(conicalGradient);

   } else {
     renderArea->setBrush(QBrush(Qt::green, style));

   }
}

DrawArea::DrawArea(QWidget *parent)
    : QWidget(parent)
{
   setBackgroundRole(QPalette::Base);
   setAutoFillBackground(true);
}

void DrawArea::setPen(const QPen &newPen)
{
   pen = newPen;
   update();
}

void DrawArea::setBrush(const QBrush &newBrush)
{
   brush = newBrush;
   update();
}

void DrawArea::paintEvent(QPaintEvent *)
{
   static const QVector<QPoint> points = {
      QPoint(30, 240), QPoint(60, 30), QPoint(240, 90), QPoint(270, 210)
   };

   QPainter painter(this);
   painter.setPen(pen);
   painter.setBrush(brush);
   painter.setRenderHint(QPainter::Antialiasing, true);

   painter.save();
   painter.translate(175, 10);
   painter.drawPolygon(points.constData(), points.size());
   painter.restore();

   painter.setRenderHint(QPainter::Antialiasing, false);
   painter.setPen(palette().dark().color());
   painter.setBrush(Qt::NoBrush);

   painter.drawRect(QRect(0, 0, width() - 1, height() - 1));
}

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   MainWindow *window = new MainWindow();
   window->show();

   return app.exec();
}
